@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2020 Timothy C. Bates
+#   Copyright 2007-2022 Timothy C. Bates
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -75,6 +75,7 @@
 #' @param report Report as markdown to the console, or open a table in browser ("html")
 #' @param summary  run [umxSummary()] on the underlying umxRAM model? (Default = FALSE)
 #' @param name A name for your model (default = efa)
+#' @param tryHard Default ('no') uses normal mxRun. "yes" uses mxTryHard. Other options: "ordinal", "search"
 #' @param digits rounding (default = 2)
 #' @param n.obs Number of observations in if covmat provided (default = NA)
 #' @param covmat Covariance matrix of data you are modeling (not implemented)
@@ -125,9 +126,10 @@
 #' 
 #' }
 umxEFA <- function(x = NULL, factors = NULL, data = NULL, scores = c("none", 'ML', 'WeightedML', 'Regression'), minManifests = NA,
-	rotation = c("varimax", "promax", "none"), return = c("model", "loadings"), report = c("markdown", "html"), summary = FALSE, name = "efa", digits = 2, n.obs = NULL, covmat = NULL){
+	rotation = c("varimax", "promax", "none"), return = c("model", "loadings"), report = c("markdown", "html"), summary = FALSE, name = "efa", digits = 2, tryHard = c("no", "yes", "ordinal", "search"), n.obs = NULL, covmat = NULL){
 	# TODO: umxEFA: Detect ordinal items and switch to DWLS?
 	rotation = xmu_match.arg(rotation, c("varimax", "promax", "none"), check = FALSE)
+	tryHard    = match.arg(tryHard)
 	scores   = match.arg(scores)
 	return   = match.arg(return)
 
@@ -184,7 +186,7 @@ umxEFA <- function(x = NULL, factors = NULL, data = NULL, scores = c("none", 'ML
 	data = umx_scale(data)
 	if(is.null(factors)){
 		stop("You need to request at least 1 latent factor, e.g.: factors = 4")
-	} else if( length(factors) == 1 && class(factors) == "numeric"){
+	} else if( length(factors) == 1 && inherits(factors, "numeric")){
 		factors = paste0("F", c(1:factors))
 	}else{
 		# factors is a list of factor names (we hope)
@@ -211,8 +213,8 @@ umxEFA <- function(x = NULL, factors = NULL, data = NULL, scores = c("none", 'ML
 	   thisManifest = manifests[i]
 	   m1$S$lbound[thisManifest, thisManifest] = 0
 	}
-	# TODO could add tryHard support
-	m1 = mxRun(m1)
+
+	m1 = umxRun(m1, tryHard= tryHard)
 
 	# ============================
 	# = Do rotation if requested =
@@ -272,6 +274,7 @@ umxFactanal <- umxEFA
 #' @references - <https://github.com/tbates/umx>, <https://tbates.github.io>
 #' @md
 #' @examples
+#' \dontrun{
 #' m1 = umxEFA(mtcars, factors = 2)
 #' x = umxFactorScores(m1, type = 'Regression', minManifests = 3)
 #' 
@@ -281,7 +284,6 @@ umxFactanal <- umxEFA
 #' hist(x$F1)
 #' plot(F1 ~ F2, data = x)
 #' 
-#' \dontrun{
 #' m1 = umxEFA(mtcars, factors = 1)
 #' x = umxFactorScores(m1, type = 'Regression', minManifests = 3)
 #' x
@@ -311,9 +313,9 @@ umxFactorScores <- function(model, type = c('ML', 'WeightedML', 'Regression'), m
 }
 
 
-#' Build a SEM implementing the equivalent of 2-stage least squares regression
+#' Build a SEM implementing the instrumental variable design
 #'
-#' `umxMR` (`umxTwoStage`) implements the Structural Equation Model equivalent of a 2SLS regression.
+#' `umxMR` (`umxTwoStage`) implements a Mendelian randomization or instrumental variable Structural Equation Model.
 #' For ease of learning, the parameters follow the `tsls()` function in the sem package.
 #' 
 #' The example is a [Mendelian Randomization](https://en.wikipedia.org/wiki/Mendelian_randomization)
@@ -329,18 +331,18 @@ umxFactorScores <- function(model, type = c('ML', 'WeightedML', 'Regression'), m
 #' @param instruments A one-sided formula specifying instrumental variables (default = qtl).
 #' @param data Frame containing the variables in the model.
 #' @param subset (optional) vector specifying a subset of observations to be used in fitting the model.
-#' @param weights (optional) vector of weights to be used in the fitting process (not supported)
-#' If specified should be a non-negative numeric vector with one entry for each observation,
-#' to be used to compute weighted 2SLS estimates.
 #' @param contrasts	an optional list (not supported)
-#' @param name for the model (default = "tsls")
+#' @param name for the model (default = "IVmodel")
+#' @param tryHard Default ('no') uses normal mxRun. "yes" uses mxTryHard. Other options: "ordinal", "search"
 #' @param ...	arguments to be passed along. (not supported)
 #' @return - [mxModel()]
 #' @export
 #' @family Super-easy helpers
 #' @seealso - [umx_make_MR_data()], [umxRAM()]
-#' @references - * Fox, J. (1979) Simultaneous equation models and two-stage least-squares. In Schuessler, K. F. (ed.) *Sociological Methodology*, Jossey-Bass.
+#' @references - Fox, J. (1979) Simultaneous equation models and two-stage least-squares. In Schuessler, K. F. (ed.) *Sociological Methodology*, Jossey-Bass.
 #' * Greene, W. H. (1993) *Econometric Analysis*, Second Edition, Macmillan.
+#' * Sekula, P., Del Greco, M. F., Pattaro, C., & Kottgen, A. (2016). Mendelian Randomization as an Approach to 
+#' Assess Causality Using Observational Data. *Journal of the American Society of Nephrology*, **27**), 3253-3265. \doi{10.1681/ASN.2016010098}
 #' @md
 #' @examples
 #' \dontrun{
@@ -351,8 +353,7 @@ umxFactorScores <- function(model, type = c('ML', 'WeightedML', 'Regression'), m
 #' # = Mendelian Randomization analysis =
 #' # ====================================
 #' 
-#' # Note: in practice: many more subjects are desirable - this just to let example run fast
-#' df = umx_make_MR_data(1000) 
+#' df = umx_make_MR_data(10e4)
 #' m1 = umxMR(Y ~ X, instruments = ~ qtl, data = df)
 #' parameters(m1)
 #' plot(m1, means = FALSE, min="") # help DiagrammaR layout the plot.
@@ -365,7 +366,7 @@ umxFactorScores <- function(model, type = c('ML', 'WeightedML', 'Regression'), m
 #' m1 = lm(Y ~ X + U, data = df); coef(m1) # Controlling U reveals the true 0.1 beta weight
 #'
 #'
-#' df = umx_make_MR_data(1e5) 
+#' df = umx_make_MR_data(10e4) 
 #' m1 = umxMR(Y ~ X, instruments = ~ qtl, data = df)
 #' coef(m1)
 #' 
@@ -379,12 +380,10 @@ umxFactorScores <- function(model, type = c('ML', 'WeightedML', 'Regression'), m
 #' # Try with missing value for one subject: A benefit of the FIML approach in OpenMx.
 #' m3 = tsls(formula = Y ~ X, instruments = ~ qtl, data = (df[1, "qtl"] = NA))
 #' }
-umxTwoStage <- function(formula= Y ~ X, instruments = ~qtl, data, subset, weights, contrasts= NULL, name = "tsls", ...) {
-	umx_check(is.null(contrasts), "stop", "Contrasts not supported yet in umxTwoStage: email maintainer('umx') to prioritize")	
-	# formula = Y ~ X; instruments ~ qtl; data = umx_make_MR_data(10000)
-	# m1 = sem::tsls(formula = Y ~ X, instruments = ~ qtl, data = df)
-	# summary(sem::tsls(Q ~ P + D, ~ D + F + A, data=Kmenta))
-	if(!class(formula) == "formula"){
+umxTwoStage <- function(formula= Y ~ X, instruments = ~qtl, data, subset, contrasts= NULL, name = "IV_model", tryHard = c("no", "yes", "ordinal", "search"), ...) {
+	# tryHard = match.arg(tryHard)
+	umx_check(is.null(contrasts), "stop", "Contrasts not supported yet in umxMR: e-mail maintainer('umx') to prioritize")	
+	if(!inherits(formula, "formula")){
 		stop("formula must be a formula")
 	}
 	allForm = all.vars(terms(formula))
@@ -398,22 +397,20 @@ umxTwoStage <- function(formula= Y ~ X, instruments = ~qtl, data, subset, weight
 		stop("I'm currently limited to 1 DV, 1 IV, and 1 instrument: 'instruments' had ", length(allForm), " items")
 	}
 	manifests <- c(allForm, inst)     # manifests <- c("qtl", "X", "Y")
-	latentErr <- paste0("e", allForm) # latentErr   <- c("eX", "eY")
+	latentErr <- paste0("e", allForm) # latentErr <- c("eX", "eY")
 	umx_check_names(manifests, data = data, die = TRUE)
 
-	IVModel = umxRAM("IV Model", data = data,
+	IVModel = umxRAM(name, data = data, tryHard = tryHard,
 		# Causal and confounding paths
 		umxPath(inst , to = Xvars), # beta of SNP effect          :  X ~ b1 x inst
 		umxPath(Xvars, to = DV),    # Causal effect of Xvars on DV: DV ~ b2 x X
-
 		# Latent error stuff + setting up variance and means for variables
 		umxPath(v.m. = inst),     # Model variance and mean of instrument
 		umxPath(var = latentErr), # Variance of residual errors
-		umxPath(latentErr, to = allForm, fixedAt = 1), # X and Y residuals@1.
+		umxPath(latentErr, to = allForm, fixedAt = 1), # X and Y residuals@1
 		umxPath(unique.bivariate = latentErr, values = 0.2, labels = paste0("phi", length(latentErr)) ), # Correlation among residuals
 		umxPath(means = c(Xvars, DV))
 	)
-	# umx_time(IVModel) # IV Model: 3.1 s ( was 14.34 seconds with poor start values) for 100,000 subjects
 	return(IVModel)
 }
 
